﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Cruiser;
using Monorail.Entities;
using Monorail.MovementStrategy;

namespace Monorail.DrawningObjects
{
    public class DrawningMonorail
    {
        public EntityMonorail? EntityMonorail { get; protected set; }

        public int GetPosX => _startPosX;

        public int GetPosY => _startPosY;

        public int GetWidth => _monoRailWidth;

        public int GetHeight => _monoRailHeight;

        private int _pictureWidth;

        private int _pictureHeight;

        protected int _startPosX = 0;

        protected int _startPosY = 0;

        protected int _monoRailWidth = 110;

        protected int _monoRailHeight = 60;

        protected int wheelSz;
        public DrawningMonorail(int speed, double weight, Color bodyColor, Color wheelColor, Color tireColor, int width, int height)
        {
            if (width <= _monoRailWidth || height <= _monoRailHeight)
                return;
            _pictureWidth = width;
            _pictureHeight = height;
            wheelSz = _monoRailHeight - _monoRailHeight * 7 / 10;
            EntityMonorail = new EntityMonorail(speed, weight, bodyColor, wheelColor, tireColor);
        }

        protected DrawningMonorail(int speed, double weight, Color bodyColor, int width, int height, Color wheelColor, Color tireColor, int monoRailWidth, int monoRailHeight)
        {
            if (width <= _monoRailWidth || height <= _monoRailHeight)
                return;
            _pictureWidth = width;
            _pictureHeight = height;
            _monoRailHeight = monoRailHeight;
            _monoRailWidth = monoRailWidth;
            wheelSz = _monoRailHeight - _monoRailHeight * 7 / 10;
            EntityMonorail = new EntityMonorail(speed, weight, bodyColor, wheelColor, tireColor);
        }
        public void SetPosition(int x, int y)
        {
            if (EntityMonorail == null)
                return;
            _startPosX = x;
            _startPosY = y;
            if (x + _monoRailWidth >= _pictureWidth || y + _monoRailHeight >= _pictureHeight)
            {
                _startPosX = 1;
                _startPosY = 1;
            }
        }
        public IMoveableObject GetMoveableObject => new
DrawningObjectMonorail(this);
        public bool CanMove(DirectionType direction)
        {
            if (EntityMonorail == null)
                return false;
            return direction switch
            {
                DirectionType.Left => _startPosX - EntityMonorail.Step > 0,
                DirectionType.Up => _startPosY - EntityMonorail.Step > 0,
                DirectionType.Right => _startPosX + EntityMonorail.Step < _pictureWidth,
                DirectionType.Down => _startPosY - +EntityMonorail.Step < _pictureHeight
            };
        }
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityMonorail == null)
                return;
            switch (direction)
            {
                case DirectionType.Left:
                    if (_startPosX - EntityMonorail.Step >= 0)
                        _startPosX -= (int)EntityMonorail.Step;
                    break;
                case DirectionType.Up:
                    if (_startPosY - EntityMonorail.Step >= 0)
                        _startPosY -= (int)EntityMonorail.Step;
                    break;
                case DirectionType.Right:
                    if (_startPosX + EntityMonorail.Step + _monoRailWidth < _pictureWidth)
                        _startPosX += (int)EntityMonorail.Step;
                    break;
                case DirectionType.Down:
                    if (_startPosY + EntityMonorail.Step + _monoRailHeight < _pictureHeight)
                        _startPosY += (int)EntityMonorail.Step;
                    break;
            }
        }

        public virtual void DrawTransport(Graphics g)
        {
            if (EntityMonorail == null)
                return;
            int dif = _monoRailWidth / 10;
            _monoRailWidth -= dif;
            Pen pen = new(Color.Black);
            Brush cartBrush = new SolidBrush(Color.Black);
            Brush bodyBrush = new SolidBrush(EntityMonorail.BodyColor);
            Pen windowPen = new(Color.Blue);
            Brush wheelBrush = new SolidBrush(EntityMonorail.WheelColor);
            Brush windowBrush = new SolidBrush(Color.White);
            Pen tirePen = new Pen(EntityMonorail.TireColor);
            if (_monoRailWidth - _monoRailWidth / 20 * 17 < wheelSz)
                wheelSz = _monoRailWidth - _monoRailWidth / 20 * 17;
            g.DrawEllipse(pen, _startPosX + 15, _startPosY + 5, 20, 20);
            g.DrawEllipse(pen, _startPosX + 15, _startPosY + 35, 20, 20);
            g.DrawRectangle(pen, _startPosX + 9, _startPosY + 15, 10, 30);
            g.DrawRectangle(pen, _startPosX + 90, _startPosY + 15, 10,
           30);
            g.DrawRectangle(pen, _startPosX + 20, _startPosY + 4, 70, 52);

            //если есть доп.фонари
            /* if (Cruiser.Headlights)
             {

                 Brush brYellow = new SolidBrush(Color.Yellow);
                 g.FillEllipse(brYellow, _startPosX + 80, _startPosY + 5, 20,
                 20);
                 g.FillEllipse(brYellow, _startPosX + 80, _startPosY + 35, 20,
                 20);
             }*/
            //основание лодки!!!
            Brush br = new SolidBrush(EntityMonorail.BodyColor);
            g.FillRectangle(br, _startPosX + 10, _startPosY + 15, 10, 30);
            g.FillRectangle(br, _startPosX + 90, _startPosY + 15, 10, 30);
            g.FillRectangle(br, _startPosX + 20, _startPosY + 5, 70, 50);
            Point[] points = new Point[3];// нос лодки
            points[0] = new Point(_startPosX + 100, _startPosY + 5);
            points[1] = new Point(_startPosX + 100, _startPosY + 55);
            points[2] = new Point(_startPosX + 100 + 50, _startPosY + 50 / 2);
            g.FillPolygon(Brushes.Pink, points);
            //границы носа лодки
            Point[] points1 = new Point[3];// нос лодки
            points1[0] = new Point(_startPosX + 100, _startPosY + 5);
            points1[1] = new Point(_startPosX + 100, _startPosY + 55);
            points1[2] = new Point(_startPosX + 100 + 50, _startPosY + 50 / 2);
            g.DrawPolygon(pen, points1);
            g.FillRectangle(Brushes.Black, _startPosX + 5, _startPosY + 15, 10, 10);
            g.FillRectangle(Brushes.Black, _startPosX + 5, _startPosY + 35, 10, 10);

            //если есть ракетные шахты, добавить условие
            g.DrawRectangle(Pens.Black, _startPosX + 35,
               _startPosY + 23, 15, 15);
            g.DrawRectangle(Pens.Black, _startPosX + 50,
            _startPosY + 19, 30, 25);





           // _monoRailWidth += dif;

        }
    }

}

