using System;
using Cruiser;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Monorail.DrawningObjects;
using Monorail.MovementStrategy;
using Monorail;


namespace Cruiser
{

    public partial class CruiserForm : Form
    {
        private DrawningMonorail? _drawningMonorail;

        private AbstractStrategy? _abstractStrategy;
        private AbstractStrategy? _strategy;

        public DrawningMonorail? SelectedCar { get; private set; }

        public CruiserForm()
        {
            InitializeComponent();
            _strategy = null;
            SelectedCar = null;
        }
        private void Draw()
        {
            if (_drawningMonorail == null)
            {
                return;
            }
            Bitmap bmp = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningMonorail.DrawTransport(gr);
            pictureBox1.Image = bmp;
        }
        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawningMonorail == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningMonorail.MoveTransport(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawningMonorail.MoveTransport(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawningMonorail.MoveTransport(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawningMonorail.MoveTransport(DirectionType.Right);
                    break;
            }
            Draw();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color wheelColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color tireColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                bodyColor = dialog.Color;
            }
            _drawningMonorail = new DrawningMonorail(random.Next(100, 300), random.Next(1000, 3000),
                bodyColor, wheelColor, tireColor,
                pictureBox1.Width, pictureBox1.Height);
            Draw();
        }


    


    private void ButtonStep_Click(object sender, EventArgs e)
        {
            if (_drawningMonorail == null)
            {
                return;
            }
            if (comboBox1.Enabled)
            {
                _abstractStrategy = comboBox1.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToEdge(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new DrawningObjectMonorail(_drawningMonorail), pictureBox1.Width,
                pictureBox1.Height);
                comboBox1.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBox1.Enabled = true;
                _abstractStrategy = null;
            }
        }

        private void buttonCreate_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color wheelColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color tireColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
                bodyColor = dialog.Color;
            if (dialog.ShowDialog() == DialogResult.OK)
                additionalColor = dialog.Color;
            int addWheelNumb = random.Next(3, 5);
            _drawningMonorail = new DrawningLocomotive(random.Next(100, 300), random.Next(1000, 3000),
                bodyColor, wheelColor, tireColor,
                pictureBox1.Width, pictureBox1.Height, addWheelNumb,
                additionalColor, random.Next(0, 2) > 0, random.Next(0, 2) > 0);
            Draw();
        }

        public void SelectedCruiser_Click(object sender, EventArgs e)
        {
            SelectedCar = _drawningMonorail;
            DialogResult = DialogResult.OK;
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {

        }
    }
}






