﻿using Monorail.MovementStrategy;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Monorail.DrawningObjects;
using System.Drawing;


namespace Monorail.Generics
{
    internal class MonorailGenericCollection<T, U>
        where T : DrawningMonorail
        where U : IMoveableObject
    {
        private readonly int _pictureWidth;

        private readonly int _pictureHeight;

        private readonly int _placeSizeWidth = 133;

        private readonly int _placeSizeHeight = 50;

        private readonly SetGeneric<T> _collection;

        public MonorailGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _collection = new SetGeneric<T>(width * height);
        }

        public static bool operator +(MonorailGenericCollection<T, U> collect, T? obj)
        {
            if (obj == null)
                return false;

            return collect?._collection.Insert(obj) ?? false;
        }

        public static T? operator -(MonorailGenericCollection<T, U> collect, int
pos)
        {
            T? obj = collect._collection[pos];
            if (obj != null)
                collect._collection.Remove(pos);
            return obj;
        }

        public U? GetU(int pos)
        {
            return (U?)_collection[pos]?.GetMoveableObject;
        }

        public Bitmap ShowMonorails()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }

        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                1; ++j)
                {
                    g.DrawLine(pen, i * _placeSizeWidth, j *
                    _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                    _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i *
                _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
            }
        }

        private void DrawObjects(Graphics g)
        {
            int i = 0;
            foreach (var monorail in _collection.GetMonorails())
            {
                if (monorail != null)
                {
                    int inRow = _pictureWidth / _placeSizeWidth;
                   // monorail.SetPosition(i % inRow * _placeSizeWidth, _pictureHeight - _pictureHeight % _placeSizeHeight - (i / inRow + 1) * _placeSizeHeight + 5);
                    monorail.SetPosition((i % inRow) * _placeSizeWidth + _placeSizeWidth / 50, _placeSizeHeight * (i / inRow) + _placeSizeHeight / 20);
                   
                    monorail.DrawTransport(g);
                }
                i++;
            }
        }

    }
}